package org.mindtask.efs.boxnet;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import cn.com.believer.songyuanframework.openapi.storage.box.functions.UploadResponse;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.BoxException;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.BoxFile;
import cn.com.believer.songyuanframework.openapi.storage.box.objects.UploadResult;

/**
 * The box.net special <code>OutputStream</code> that flush content to remote box.net while calling {@link #close()}
 * 
 * @author tanxu
 * @date Jan 22, 2012
 * @since 1.0
 */
public class BoxOutputStream extends OutputStream {

	private ByteArrayOutputStream bos;
	private BoxFileStore fileStore;

	public BoxOutputStream(BoxFileStore fileStore) {
		bos = new ByteArrayOutputStream();
		this.fileStore = fileStore;
	}

	@Override
	public void write(int b) throws IOException {
		bos.write(b);
	}

	@Override
	public void write(byte[] b) throws IOException {
		bos.write(b);
	}

	@Override
	public void write(byte[] b, int off, int len) throws IOException {
		bos.write(b, off, len);
	}

	@Override
	public void close() throws IOException {
		bos.close();
		try {
			UploadResponse uploadResponse = fileStore.getFileSystem().uploadFile(fileStore, bos.toByteArray());
			BoxFile boxFile = ((UploadResult) uploadResponse.getUploadResultList().get(0)).getFile();
			fileStore.setBoxFile(boxFile);
			// TODO set the parent id?
			BoxFileInfo info = fileStore.fetchInfo();
			info.setExists(true);
		}
		catch (BoxException e) {
			throw new IOException("Failed to upload content to file [" + fileStore.getName() + "]: " + e.getMessage(),
					e);
		}
	}
}
