package org.mindtask.efs.boxnet;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.runtime.Path;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import com.google.common.io.ByteStreams;

/**
 * DOCME
 * 
 * @author tanxu
 * @date Jan 21, 2012
 * @since
 */
public class BoxFileSystemTest {

	private static BoxFileSystem bfs;
	private static IFileStore rootFileStore;
	private static IFileStore testFolderStore;

	/**
	 * the url in form of <code>boxnet://[username]:[auth_token]@box.net/[path] <br/>
	 * for unit test code, it would be better to hard-coding the auth token here, 
	 * since box.net does something special to protect from log in with simulating request, 
	 * for exmaple, sending http request with httpclient won't success 
	 */
	private static String url = "boxnet://missedone:2qttcnhou36fg5apik306gk52hj5bvre@box.net";
	
	@BeforeClass
	public static void setUp() throws Exception {
		bfs = (BoxFileSystem) EFS.getFileSystem(org.mindtask.efs.boxnet.BoxFileSystem.SCHEME_BOXNET);
		URI uri = new URI(url);
		rootFileStore = bfs.getStore(uri);
		assertTrue(rootFileStore.fetchInfo().isDirectory());
		assertTrue(rootFileStore instanceof BoxFileStore);

		testFolderStore = rootFileStore.getChild("Test");
		assertNotNull(testFolderStore);
		IFileStore parentFileStore = testFolderStore.getParent();
		assertNotNull(parentFileStore);
		assertEquals("0", ((BoxFileStore) parentFileStore).getBoxFile().getId()); // the parent should be root

		if (!testFolderStore.fetchInfo().exists()) {
			testFolderStore = testFolderStore.mkdir(EFS.NONE, null);
		}
		assertNotNull(testFolderStore);
		assertTrue(testFolderStore.fetchInfo().exists());

		uri = new URI("boxnet://missedone:2qttcnhou36fg5apik306gk52hj5bvre@box.net/Test");
		IFileStore testFolderStore2 = bfs.getStore(uri);
		assertNotNull(testFolderStore2);
		assertNotNull(testFolderStore2.getParent());

		IFileStore testFileStore = testFolderStore.getChild("readme.txt");
		assertFalse(testFileStore.fetchInfo().exists());
		assertNotNull(testFileStore.getParent());
	}

	@AfterClass
	public static void tearDown() throws Exception {
		IFileStore testFileStore = testFolderStore.getChild("readme.txt");
		if (testFileStore.fetchInfo().exists()) {
			testFileStore.delete(EFS.NONE, null);
		}
		testFileStore = testFolderStore.getChild("readme.txt");
		assertFalse(testFileStore.fetchInfo().exists());
	}

	@Test
	public void testGetFileStore() throws Exception {
		IFileStore[] childStores = testFolderStore.childStores(EFS.NONE, null);
		int childCnt = childStores.length;
		IFileStore testFileStore = testFolderStore.getChild("readme.txt");
		OutputStream out = testFileStore.openOutputStream(EFS.NONE, null);
		assertNotNull("open outputStream return null", out);
		BufferedOutputStream bos = new BufferedOutputStream(out);
		String strContent = "hello, world!";
		bos.write(strContent.getBytes());
		bos.close(); // close to flush the content to box.net
		assertTrue("new file should exist", testFileStore.fetchInfo().exists());
		assertEquals(((BoxFileStore) testFileStore).getBoxFile().getParentId(), ((BoxFileStore) testFolderStore)
				.getBoxFile().getId());

		Thread.sleep(10000); // sleep to wait uploading complete

		childStores = testFolderStore.childStores(EFS.NONE, null);
		assertEquals(childCnt + 1, childStores.length);

		testFileStore = rootFileStore.getFileStore(new Path("Test/readme.txt"));
		assertNotNull(testFileStore);
		assertTrue("new file should exist", testFileStore.fetchInfo().exists());

		assertEquals("boxnet://missedone:2qttcnhou36fg5apik306gk52hj5bvre@box.net/Test/readme.txt", testFileStore
				.toURI().toString());

		InputStream input = testFileStore.openInputStream(EFS.NONE, null);
		assertNotNull(input);
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ByteStreams.copy(input, baos);
		assertEquals(strContent, new String(baos.toByteArray()));
	}

}
